var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var app;
(function (app) {
    /// <reference path="../services/utilities.ts" />
    /// <reference path="../services/utilities.ts" />
    /// <reference path="../controllers/appController.ts" />
    /// <reference path="../../../Scripts/typings/angularjs/angular.d.ts" />
    /// <reference path="commonDirectives.ts" />
    (function (directives) {
        var RequiredDirective = (function (_super) {
            __extends(RequiredDirective, _super);
            function RequiredDirective() {
                _super.call(this);
                this.restrict = 'A';
                this.require = ['?ngModel'];
                var that = this;
                this.link = function (scope, element, attributes, controller) {
                    var currentController = app.directives.BaseDirective.getControllerFromParameterArray(controller);
                    if (!currentController)
                        return;
                    var validator = function (value) {
                        if ((that.isEmpty(value) || value === false)) {
                            currentController.$setValidity('required', false);
                            return value;
                        } else {
                            currentController.$setValidity('required', true);
                            return value;
                        }
                    };

                    currentController.$formatters.push(validator);
                    currentController.$parsers.unshift(validator);
                };
            }
            return RequiredDirective;
        })(directives.BaseDirective);

        var MaxLengthDirective = (function (_super) {
            __extends(MaxLengthDirective, _super);
            function MaxLengthDirective() {
                _super.call(this);
                this.restrict = 'A';
                this.require = ['?ngModel'];
                var that = this;
                this.link = function (scope, element, attributes, controller) {
                    var maxLength = parseInt(element.attr(attributes.$attr['maxlen']));
                    var currentController = app.directives.BaseDirective.getControllerFromParameterArray(controller);
                    if (!currentController)
                        return;
                    var validator = function (value) {
                        if (!that.isEmpty(value) && value.length > maxLength) {
                            currentController.$setValidity('maxlen', false);
                            return value;
                        } else {
                            currentController.$setValidity('maxlen', true);
                            return value;
                        }
                    };

                    currentController.$formatters.push(validator);
                    currentController.$parsers.unshift(validator);
                };
            }
            return MaxLengthDirective;
        })(directives.BaseDirective);

        var DropdownRequiredDirective = (function (_super) {
            __extends(DropdownRequiredDirective, _super);
            function DropdownRequiredDirective() {
                _super.call(this);
                this.restrict = 'A';
                this.require = ['?ngModel'];
                var that = this;
                this.link = function (scope, element, attributes, controller) {
                    var currentController = app.directives.BaseDirective.getControllerFromParameterArray(controller);
                    if (!currentController)
                        return;
                    var validator = function (value) {
                        if (that.isEmpty(value)) {
                            currentController.$setValidity('dropdownRequired', false);
                            return value;
                        } else {
                            var intValue = parseInt(value);
                            if (intValue) {
                                currentController.$setValidity('dropdownRequired', true);
                            } else {
                                currentController.$setValidity('dropdownRequired', false);
                            }
                            return value;
                        }
                    };

                    currentController.$formatters.push(validator);
                    currentController.$parsers.unshift(validator);
                };
            }
            return DropdownRequiredDirective;
        })(directives.BaseDirective);

        var FormSubmitDirectiveFunctionName = 'formSubmitFunction';
        directives.FormSubmitDirectiveName = 'formSubmit';

        var FormSubmitDirective = (function (_super) {
            __extends(FormSubmitDirective, _super);
            function FormSubmitDirective(utilities) {
                _super.call(this);
                this.utilities = utilities;
                this.restrict = 'A';
                this.link = function (scope, element, attributes) {
                    var $element = angular.element(element);

                    $element.bind('click', function (eventObject) {
                        eventObject.preventDefault();

                        var form = scope.$eval(attributes[directives.FormSubmitDirectiveName]);

                        // trigger all pristine fields to be dirty and run validation
                        angular.forEach(form, function (fieldController) {
                            if (fieldController.$pristine) {
                                fieldController.$setViewValue(fieldController.$viewValue);
                            }
                        });

                        if (form.$valid) {
                            // call function defined in another attribute
                            scope.$eval(attributes[FormSubmitDirectiveFunctionName]);
                        } else {
                            // apply scope changes to show error messages
                            scope.$apply();
                            utilities.showMessage('Please correct invalid values.');
                        }
                    });
                };
            }
            return FormSubmitDirective;
        })(directives.BaseDirective);

        angular.module('app.directives.validation', []).directive('required', [
            function () {
                return new RequiredDirective();
            }
        ]).directive('maxlen', [
            function () {
                return new MaxLengthDirective();
            }
        ]).directive('dropdownRequired', [
            function () {
                return new DropdownRequiredDirective();
            }
        ]).directive('formSubmit', [
            'utilities',
            function (utilities) {
                return new FormSubmitDirective(utilities);
            }
        ]);
    })(app.directives || (app.directives = {}));
    var directives = app.directives;
})(app || (app = {}));
//# sourceMappingURL=validationDirectives.js.map
