var app;
(function (app) {
    (function (core) {
        (function (services) {
            /// <reference path="../../../Scripts/typings/angularjs/angular.d.ts" />
            /// <reference path="../../home/interfaces.ts" />
            (function (http) {
                
                

                var Http = (function () {
                    function Http($http, utilities, globalsService) {
                        this.$http = $http;
                        this.utilities = utilities;
                        this.globalsService = globalsService;
                        var that = this;
                        this.post = function (urlFragment, data, callback, showPleaseWait) {
                            var url = that.globalsService.webApiBaseUrl + urlFragment;
                            var promise = that.$http.post(url, data, { showPleaseWait: showPleaseWait });
                            promise.error(function (result, status, headers, config) {
                                if (status !== 0) {
                                    that.utilities.showMessage('Error getting server data.');
                                    callback({ Result: null, Success: false, ErrorMessage: result });
                                }
                            });
                            promise.success(function (result, status, headers, config) {
                                if (!result.Success) {
                                    that.utilities.showMessage(result.ErrorMessage);
                                }
                                callback(result);
                            });
                            return promise;
                        };

                        this.get = function (urlFragment, callback, showPleaseWait, params) {
                            var url = that.globalsService.webApiBaseUrl + urlFragment;
                            var promise;
                            if (params) {
                                promise = that.$http.get(url, { params: params, showPleaseWait: showPleaseWait });
                            } else {
                                promise = that.$http.get(url, { showPleaseWait: showPleaseWait });
                            }
                            promise.error(function (result, status, headers, config) {
                                if (status !== 0) {
                                    that.utilities.showMessage('Error getting server data.');
                                    callback({ Result: null, Success: false, ErrorMessage: result });
                                }
                            });
                            promise.success(function (result, status, headers, config) {
                                if (!result.Success) {
                                    that.utilities.showMessage(result.ErrorMessage);
                                }
                                callback(result);
                            });
                            return promise;
                        };
                    }
                    return Http;
                })();

                var HttpInterceptor = (function () {
                    function HttpInterceptor($q, utilities, globalsService) {
                        this.$q = $q;
                        this.utilities = utilities;
                        this.globalsService = globalsService;
                        var totalRequests = 0;
                        var that = this;
                        this.request = function (config) {
                            if (config.showPleaseWait) {
                                totalRequests++;
                                if (totalRequests === 1) {
                                    that.utilities.showPleaseWait();
                                }
                            }
                            return config;
                        };
                        this.requestError = function (rejection) {
                            var config = rejection['config'];
                            if (config && config.showPleaseWait) {
                                totalRequests--;
                                if (totalRequests === 0) {
                                    that.utilities.hidePleaseWait();
                                }
                            }
                            return $q.reject(rejection);
                        };
                        this.response = function (response) {
                            var config = response['config'];
                            if (config && config.showPleaseWait) {
                                totalRequests--;
                                if (totalRequests === 0) {
                                    that.utilities.hidePleaseWait();
                                }
                            }
                            return response;
                        };
                        this.responseError = function (rejection) {
                            var config = rejection['config'];
                            if (config && config.showPleaseWait) {
                                totalRequests--;
                                if (totalRequests === 0) {
                                    that.utilities.hidePleaseWait();
                                }
                            }
                            return $q.reject(rejection);
                        };
                    }
                    return HttpInterceptor;
                })();

                angular.module('app.core.services.http', ['app.core.services.utilities', 'app.globalsModule']).config([
                    '$httpProvider',
                    function ($httpProvider) {
                        $httpProvider.interceptors.push('httpInterceptor');
                    }
                ]).factory('http', [
                    '$http',
                    'utilities',
                    'globalsService',
                    function ($http, utilities, globalsService) {
                        return new Http($http, utilities, globalsService);
                    }
                ]).factory('httpInterceptor', [
                    '$q',
                    'utilities',
                    'globalsService',
                    function ($q, utilities, globalsService) {
                        return new HttpInterceptor($q, utilities, globalsService);
                    }
                ]);
            })(services.http || (services.http = {}));
            var http = services.http;
        })(core.services || (core.services = {}));
        var services = core.services;
    })(app.core || (app.core = {}));
    var core = app.core;
})(app || (app = {}));
//# sourceMappingURL=http.js.map
